<?php
	/**
	 * @property NetsuiteConnection $netsuite_connection
	 */
	class insertorupdatecategory extends WC_API_Resource
	
	{
		
		protected $base = '/insertorupdatecategory';
		public function register_routes($routes)
		{
			
			# GET /getcategorynsids
			
			$routes[ $this->base ] = [
				[[$this, 'get_data_iouc'], WC_API_Server::READABLE | WC_API_SERVER::EDITABLE | WC_API_SERVER::ACCEPT_DATA],
			];
			
			return $routes;
			
		}
		
		
		public function get_data_iouc()
		{
			//echo "cat post ".$_POST["categories"]."\n\n";
			$this->netsuite_init(0);
			
			//print_r($_POST);
			$this->netsuite_connection->processMessage("Categories sync started.");
			echo "Endpoint called.\n\n";
			$nscategories = json_decode(base64_decode($_REQUEST["categories"]), TRUE);
			if (!isset($nscategories)) {
				$message = "Missing data argument.";
				$this->netsuite_connection->processMessage($message, true, true);
				echo json_encode(["errors" => $message]);
				exit;
			}
			
			
			// get the categories internal ids
			$internalIds = implode(', ', array_filter(array_map(function($a){ return @$a['internalid']; }, $nscategories)));
			$this->netsuite_connection->processMessage("We are going to create/update categories with following internalIds <strong>{$internalIds}</strong>.");
			
			do_action('in8_category_sync_before', $nscategories);
			
			
			$nscategories = apply_filters('in8_category_sync_categories', $nscategories);
			foreach ($nscategories as $cat) {
				// collect the category info
				$taxonomy_name = isset($cat['taxonomy']) ? $cat['taxonomy'] : 'product_cat';
				$nsiid = $cat["internalid"];
				$nsname = $cat["name"];
				$nsmetahtml = ($cat["metahtml"]);
				$nsdescription = ($cat["description"]);
				$nskeywords = $cat["keywords"];
				$nsthumbnail = $cat["thumbnail"];
				$nsimage = $cat["image"];
				$nsdetailed_description = $cat["detailed_description"];
				$nsbrief_description = $cat["brief_description"];
				$nsurl = $cat["urlcomponent"];
				$nsparent = $cat["parent"];
				$wc_cat_parent_id = NULL;
				
				if( $cat['is_attribute'] ){
					$taxonomy_name = 'pa_'.wc_sanitize_taxonomy_name($taxonomy_name);
				}
				
				if( !taxonomy_exists($taxonomy_name ) ){
					// TODO create taxonomy if it's set to
					if( isset($cat['is_attribute']) && strtolower($cat['is_attribute']) == 't' ){
						// create the new attribute
						$result = $this->create_attribute($cat);
						
						// if the attribute was created then update the taxonomy to the correct taxonomy
						if( $result != false ){
							$taxonomy_name = $cat['taxonomy'] = $result['taxonomy'];
						}else{
							$message = "Couldn't create attribute `{$taxonomy_name}`.";
							$this->netsuite_connection->processMessage($message, TRUE, TRUE);
							die(json_encode([
								'errors' => $message
							]));
						}
					}else {
						$message = "Taxonomy `{$taxonomy_name}` doesn't exist.";
						$this->netsuite_connection->processMessage($message, TRUE, TRUE);
						die(json_encode([
							'errors' => $message
						]));
					}
				}
				
				
				global $wpdb;
				$table_name_nscats = $wpdb->prefix . "termmeta";
				$table_name_taxonomy = $wpdb->prefix . "term_taxonomy";
				
				$wc_cat_id = $wpdb->get_results("SELECT term_id FROM {$table_name_nscats} WHERE meta_value = '{$nsiid}'");
				if( empty($nsiid) ){ // if the nsid it's empty it means we don't have a category created properly
					$wc_cat_id = null;
				}
				
				
				// get the parent id if we have one
				if ($nsparent != NULL) {
					echo "Has a Parent\n";
					$wc_cat_parent_id = $wpdb->get_results("SELECT term_id FROM $table_name_nscats WHERE meta_value = $nsparent");
				}
				if ($wc_cat_parent_id == NULL) {
					$wc_cat_parent_id = 0;
				} else {
					$wc_cat_parent_id = $wc_cat_parent_id[0]->term_id;
				}
				
				$action = 'created';
				if ($wc_cat_id == NULL) {
					// if category_id was not included and category does not exist, insert it
					$newcat = [
						'name'        => $nsname,
						'description' => $nsdescription,
						'slug'        => $nsurl,
						'parent'      => $wc_cat_parent_id,
					];
					$insert = wp_insert_term($nsname, $taxonomy_name, $newcat);
					if( is_wp_error($insert) ){
						/* @var WP_Error $insert */
						$message = "#{$nsiid}: ".$insert->get_error_message();
						$this->netsuite_connection->processMessage($message, true, true);
						echo json_encode(["errors" => $message]);
						return;
					}
					
					$termid = $insert['term_id'];
					
					update_term_meta($termid, 'metahtml', $nsmetahtml, TRUE);
					update_term_meta($termid, 'description', $nsdescription, TRUE);
					update_term_meta($termid, 'keywords', $nskeywords, TRUE);
					update_term_meta($termid, 'internalid', $nsiid, TRUE);
					update_term_meta($termid, 'brief_description', $nsbrief_description, TRUE);
					update_term_meta($termid, 'detailed_description', $nsdetailed_description, TRUE);
					
					// If value of image is numeric, assume value as image_id.
					$image = $nsimage;
					$image_id = 0;
					if (is_numeric($image)) {
						$image_id = absint($image);
					} else if (!empty($image)) {
						$upload = upload_product_category_image(esc_url_raw($image));
						$image_id = set_uploaded_image_as_attachment($upload);
					}
					
					update_term_meta($termid, 'display_type', 'default');
					
					// Check if image_id is a valid image attachment before updating the term meta.
					if ($image_id && wp_attachment_is_image($image_id)) {
						update_term_meta($termid, 'thumbnail_id', $image_id);
					}
					
					// update the category description
					$nsdescription = $wpdb->_real_escape($nsdescription);
					$wpdb->get_results("update $table_name_taxonomy set description = '$nsdescription' WHERE term_id = $termid and taxonomy = '{$taxonomy_name}'");
				} else {
					$action = 'updated';
					$termid = $wc_cat_id[0]->term_id;
					// if category_id was included, update the category info
					$updatecat = [
						'name'        => $nsname,
						'description' => ($nsdescription),
						'slug'        => $nsurl,
						'parent'      => $wc_cat_parent_id,
					];
					
					$update = wp_update_term($termid, $taxonomy_name, $updatecat);
					if( is_wp_error($update) ){
						/* @var WP_Error $insert */
						$message = "#{$nsiid}: ".$update->get_error_message();
						$this->netsuite_connection->processMessage($message, true, true);
						echo json_encode(["errors" => $message]);
						return;
					}
					
					// set the term meta
					update_term_meta($termid, 'metahtml', $nsmetahtml, TRUE);
					update_term_meta($termid, 'description', $nsdescription, TRUE);
					update_term_meta($termid, 'keywords', $nskeywords, TRUE);
					update_term_meta($termid, 'brief_description', $nsbrief_description, TRUE);
					update_term_meta($termid, 'detailed_description', $nsdetailed_description, TRUE);
					
					if (isset($nsimage)) {
						$image_id = 0;
						
						// If value of image is numeric, assume value as image_id.
						$image = $nsimage;
						if (is_numeric($image)) {
							$image_id = absint($image);
						} else if (!empty($image)) {
							$upload = upload_product_category_image(esc_url_raw($image));
							$image_id = set_uploaded_image_as_attachment($upload);
						}
						// In case client supplies invalid image or wants to unset category image.
						if (!wp_attachment_is_image($image_id)) {
							$image_id = '';
						}
						update_term_meta($termid, 'display_type', 'default');
						// update thumbnail
						if (isset($image_id)) {
							update_term_meta($termid, 'thumbnail_id', $image_id);
						}
					}
					
					// update term description
					$nsdescription = $wpdb->_real_escape($nsdescription);
					$wpdb->get_results("update $table_name_taxonomy set description = '$nsdescription' WHERE term_id = $termid and taxonomy = '{$taxonomy_name}'");
					
				}
				$this->netsuite_connection->processMessage("Category {$action} with wc#<strong>{$termid}</strong> and ns#<strong>{$nsiid}</strong>.");
				
				if( !empty($cat['translations']) && is_array($cat['translations']) ) {
					$this->handle_translations($termid, $taxonomy_name, $cat['translations'], $cat);
				}
			}
			
			$this->netsuite_connection->processMessage("Categories sync completed.", true);
			echo json_encode(["success" => "true"]);
			exit;
			
		}
		public function handle_translations($term_id, $taxonomy, $translations, $cat){
			global $sitepress;
			
			$this->netsuite_connection->processMessage("Translation sync start.", false, false, true);
			
			// get the languages as a string
			$langs = implode(', ', array_filter(array_map(function($a){ return @$a['locale']; }, $translations)));
			$this->netsuite_connection->processMessage("We are going to create/update translations for <strong>{$langs}</strong>.");
			
			$term = get_term($term_id);
			$term_tax_id = $term->term_taxonomy_id;
			foreach($translations as $info){
				$lang = $info['locale'];
				$lang = $this->get_lang_code($lang);
				
				// check if we want to continue
				if( apply_filters('in8_category_sync_wpml_trans_skip', false, $info, $lang, $term_tax_id, $info) ){
					continue;
				}


				if( $lang == false ){
					$this->netsuite_connection->processMessage("<strong>Language `{$info['locale']}` is not set up on woocommerce.</strong>", false, true);
					continue;
				}
				
				$translation_id = apply_filters('wpml_object_id', $term_id, $taxonomy, false, $lang);
				
				if( $term->name == $info['displayname'] ){
					$info['displayname'] = '@'.$lang.' '.$info['displayname'];
				}
				$infoPost = [
					'name'        => $info['displayname'],
					'description' => $info['salesdescription'],
					'slug'			=> $info['displayname']
				];
				
				$infoPost = apply_filters('in8_product_sync_wpml_trans_post_data', $infoPost, $translation_id); // translation_id will be false on create
				$action = 'created';
				if( !$translation_id ) {
					remove_action( 'create_term', array( $sitepress, 'create_term' ), 1 );
					$translation_id = wp_insert_term($info['displayname'], $taxonomy, $infoPost);
					if( is_wp_error($translation_id) ){
						/* @var WP_Error $insert */
						$message = $translation_id->get_error_message();
						$this->netsuite_connection->processMessage($message, false, true);
						continue;
//						echo json_encode(["errors" => $message]);
//						return;
					}
					$translation_term = $translation_id;
				}else{
					$action = 'updated';
					$update = wp_update_term($translation_id, $taxonomy, ['name'=>rand(0,100), 'slug' => rand(0,100)]);
					$update = wp_update_term($translation_id, $taxonomy, $infoPost);
					if( is_wp_error($update) ){
						/* @var WP_Error $insert */
						$message = $update->get_error_message();
						$this->netsuite_connection->processMessage($message, false, true);
						continue;
//						echo json_encode(["errors" => $update->get_error_message()]);
//						return;
					}
					$translation_term = $update;
				}
				$translation_id = $translation_term['term_taxonomy_id'];
				
				$thumbnail_id = get_term_meta($term->term_id, 'thumbnail_id', true);
				if( !empty($thumbnail_id) ){
					update_term_meta($translation_term['term_id'], 'thumbnail_id', $thumbnail_id);
				}
				
				
				// Collect translation informations
				$get_language_args = ['element_id' => $term_tax_id, 'element_type' => $taxonomy];
				$wpml_element_type = apply_filters('wpml_element_type', $taxonomy);
				$original_post_language_info = apply_filters('wpml_element_language_details', NULL, $get_language_args);
				$set_language_args = [
					'element_id'           => $translation_id,
					'element_type'         => $wpml_element_type,
					'trid'                 => $original_post_language_info->trid,
					'language_code'        => $lang,
					'source_language_code' => $original_post_language_info->language_code,
				];


				$trid = $sitepress->set_element_language_details($set_language_args['element_id'], $set_language_args['element_type'], $set_language_args['trid'], $set_language_args['language_code'], $set_language_args['source_language_code']);
				if( empty($trid) ){
					$this->netsuite_connection->processMessage("<strong>The translation was created but couldn't be setup, id# {$translation_id}</strong>", false, true);
					continue;
				}
				
				$this->netsuite_connection->processMessage("Translation {$action} for <strong>{$info['locale']}</strong>");
				
				// runs after the category synced
				do_action('in8_category_sync_wpml_trans_saved', $translation_term, $trid);
			}
			$this->netsuite_connection->processMessage("Translations sync completed!", false, false, false, true);
		}
		
		/**
		 * Get language code conversion from netsuite to wpml
		 *
		 * @param $code
		 *
		 * @return bool
		 */
		function get_lang_code($code){
			global $sitepress;
			$active_langs = $sitepress->get_active_languages();
			foreach($active_langs as $lang){
				if( $lang['default_locale'] == $code ){
					return $lang['code'];
				}
			}
			return false;
		}
		
		
		public $netsuite_connection = null;
		public function netsuite_init($post_id)
		{
			
			if (!class_exists('NetsuiteConnection')) { //in case of ajax
				//netsuite api connection file
				require_once(WP_PLUGIN_DIR . '/in8sync/code/netsuiteconnection.php');
			}
			
			//netsuite connection object
			$this->netsuite_connection = new NetsuiteConnection($post_id, 'categories');
		}
		
		public function create_attribute($cat){
			global $wpdb;
			
			$attribute = array(
				'attribute_label'   => $cat['taxonomy'],
				'attribute_type'    => isset($cat['attribute_type']) ? $cat['attribute_type'] : 'select',
				'attribute_orderby' => 'menu_order',
				'attribute_public'  => 0
			);
			$attribute['attribute_name'] = wc_sanitize_taxonomy_name($attribute['attribute_label']);
			
			$exists = $wpdb->get_var(
								$wpdb->prepare(
									"SELECT attribute_id FROM " . $wpdb->prefix . "woocommerce_attribute_taxonomies
									WHERE attribute_name = %s LIMIT 1",
									$attribute['attribute_name']
								)
							);
			
			// it already exists
			if( $exists > 0 ){
				return ['taxonomy' => 'pa_'.$attribute['attribute_name']];
			}
			
			$wpdb->insert( $wpdb->prefix . 'woocommerce_attribute_taxonomies', $attribute );
			
			$_POST[ 'wcml-is-translatable-attr' ] = 1;
			do_action( 'woocommerce_attribute_added', $wpdb->insert_id, $attribute );
			delete_transient('wc_attribute_taxonomies');
			
			$taxonomy_name = 'pa_'.$attribute['attribute_name'];
			
			// when first creating the new attribute the list of attributes isn't yet refresh, this trick will allow us to attach options to the attribute without reloading the page
			if( !taxonomy_exists($taxonomy_name) ){
				register_taxonomy(
					$taxonomy_name,
					array( 'product' ),
					array(
						'public' => true,
						'labels' => array(
							'name' => __( $attribute['attribute_label'] ),
							'singular_name' => __( $attribute['attribute_label'] )
						),
					)
				);
			}
			
			return ['taxonomy' => 'pa_'.$attribute['attribute_name']];
		}
	}
	
	function set_uploaded_image_as_attachment($upload, $id = 0)
	{
		$info = wp_check_filetype($upload['file']);
		$title = '';
		$content = '';
		
		if ($image_meta = @wp_read_image_metadata($upload['file'])) {
			if (trim($image_meta['title']) && !is_numeric(sanitize_title($image_meta['title']))) {
				$title = $image_meta['title'];
			}
			if (trim($image_meta['caption'])) {
				$content = $image_meta['caption'];
			}
		}
		
		$attachment = [
			'post_mime_type' => $info['type'],
			'guid'           => $upload['url'],
			'post_parent'    => $id,
			'post_title'     => $title,
			'post_content'   => $content,
		];
		
		$attachment_id = wp_insert_attachment($attachment, $upload['file'], $id);
		if (!is_wp_error($attachment_id)) {
			wp_update_attachment_metadata($attachment_id, wp_generate_attachment_metadata($attachment_id, $upload['file']));
		}
		
		return $attachment_id;
	}
	
	function upload_product_category_image($image_url)
	{
		return upload_image_from_url($image_url, 'product_category_image');
	}
	
	function upload_image_from_url($image_url, $upload_for = 'product_image')
	{
		$file_name = basename(current(explode('?', $image_url)));
		$wp_filetype = wp_check_filetype($file_name, NULL);
		$parsed_url = @parse_url($image_url);
		
		// Check parsed URL.
		if (!$parsed_url || !is_array($parsed_url)) {
			throw new WC_API_Exception('woocommerce_api_invalid_' . $upload_for, sprintf(__('Invalid URL %s', 'woocommerce'), $image_url), 400);
		}
		
		// Ensure url is valid.
		$image_url = str_replace(' ', '%20', $image_url);
		
		// Get the file.
		$response = wp_safe_remote_get($image_url, [
			'timeout' => 10,
		]);
		
		if (is_wp_error($response) || 200 !== wp_remote_retrieve_response_code($response)) {
			throw new WC_API_Exception('woocommerce_api_invalid_remote_' . $upload_for, sprintf(__('Error getting remote image %s', 'woocommerce'), $image_url), 400);
		}
		
		// Ensure we have a file name and type.
		if (!$wp_filetype['type']) {
			$headers = wp_remote_retrieve_headers($response);
			if (isset($headers['content-disposition']) && strstr($headers['content-disposition'], 'filename=')) {
				$disposition = end(explode('filename=', $headers['content-disposition']));
				$disposition = sanitize_file_name($disposition);
				$file_name = $disposition;
			} elseif (isset($headers['content-type']) && strstr($headers['content-type'], 'image/')) {
				$file_name = 'image.' . str_replace('image/', '', $headers['content-type']);
			}
			unset($headers);
		}
		
		// Upload the file.
		$upload = wp_upload_bits($file_name, '', wp_remote_retrieve_body($response));
		
		if ($upload['error']) {
			throw new WC_API_Exception('woocommerce_api_' . $upload_for . '_upload_error', $upload['error'], 400);
		}
		
		// Get filesize.
		$filesize = filesize($upload['file']);
		
		if (0 == $filesize) {
			@unlink($upload['file']);
			unset($upload);
			throw new WC_API_Exception('woocommerce_api_' . $upload_for . '_upload_file_error', __('Zero size file downloaded', 'woocommerce'), 400);
		}
		
		unset($response);
		
		do_action('woocommerce_api_uploaded_image_from_url', $upload, $image_url, $upload_for);
		
		return $upload;
	}

