<?php
	/**
	 * @property NetsuiteConnection $netsuite_connection
	*/
	class wc_create_update_product_translation
	{
		public $netsuite_connection = null;
		
		public function __construct()
		{
			// hook for when a product is created from api
			// hook for when a product is created from api
			add_action('woocommerce_api_create_product', [$this, 'api_product'], 10, 2);
			// hook for when a product is update from api
			add_action('woocommerce_api_edit_product', [$this, 'api_product'], 10, 2);
			
		}
		
		public function netsuite_init($post_id)
		{
			
			if (!class_exists('NetsuiteConnection')) { //in case of ajax
				//netsuite api connection file
				require_once(WP_PLUGIN_DIR . '/in8sync/code/netsuiteconnection.php');
			}
			
			//netsuite connection object
			$this->netsuite_connection = new NetsuiteConnection($post_id, 'product wpml');
		}
		
		public function api_product($id, $data)
		{
			if ( ! isset($data['translations'])) return;
			
			global $sitepress;
			$translations = $data['translations'];
			
			$this->netsuite_init($id);
			
			$translations = apply_filters('in8_product_sync_wpml_trans_data', $translations, $id, $data);
			
			$this->netsuite_connection->processMessage('Product wpml translations sync start!');
			
			
			// get the languages as a string
			$langs = implode(', ', array_filter(array_map(function($a){ return @$a['locale']; }, $translations)));
			$this->netsuite_connection->processMessage("We are going to create/update translations for <strong>{$langs}</strong>.");
			
			// the original product will be updated here so we need to make sure we're not sending it to netsuite right here
			add_filter('in8_product_sync_skip', [$this, 'skip_product'], 10, 2);
			foreach ($translations as $info) {
				$lang = $info['locale'];
				$lang = $this->get_lang_code($lang);
				
				// check if we want to continue
				if( apply_filters('in8_product_sync_wpml_trans_skip', false, $info, $lang, $id, $data) ){
					continue;
				}
				
				
				if( $lang == false ){
					// TODO error notification this language it's not enabled
					$this->netsuite_connection->processMessage("<strong>Language `{$info['locale']}` is not set up on woocommerce.</strong>", false, true);
					continue;
				}
				
				$translation_id = apply_filters( 'wpml_object_id', $id, 'product', false, $lang );
				
				$infoPost = [
					'post_title'   => $info['displayname'],
					'post_content' => $info['salesdescription'],
					'post_content' => !empty($info['salesdescription']) ? $info['salesdescription'] : ' ',
					'post_excerpt' => !empty($info['storedescription']) ? $info['storedescription'] : ' ',
					'post_type'    => 'product',
					'post_status'  => 'publish',
				];
				
				$infoPost = apply_filters('in8_product_sync_wpml_trans_post_data', $infoPost, $translation_id); // translation_id will be false on create
				
				if( !$translation_id ) {
					$translation_id = wp_insert_post($infoPost);
				}else{
					$infoPost['ID'] = $translation_id;
					wp_update_post($infoPost);
				}
				
				if( !$translation_id ){
					// TODO error notification the translation product couldn't create
					$this->netsuite_connection->processMessage("<strong>The translation for language `{$info['locale']}` couldn't be created</strong>", false, true);
					continue;
				}
				
				// featured image
				$attachment_id = get_post_thumbnail_id($id);
				if( $attachment_id ) {
					set_post_thumbnail($translation_id, $attachment_id);
				}

				// Collect translation informations
				$get_language_args = ['element_id' => $id, 'element_type' => 'product'];
				$wpml_element_type = apply_filters('wpml_element_type', 'product');
				$original_post_language_info = apply_filters('wpml_element_language_details', NULL, $get_language_args);
				$set_language_args = [
					'element_id'           => $translation_id,
					'element_type'         => $wpml_element_type,
					'trid'                 => $original_post_language_info->trid,
					'language_code'        => $lang,
					'source_language_code' => $original_post_language_info->language_code,
				];
				

//				do_action('wpml_set_element_language_details', $set_language_args);
				$trid = $sitepress->set_element_language_details($set_language_args['element_id'], $set_language_args['element_type'], $set_language_args['trid'], $set_language_args['language_code'], $set_language_args['source_language_code']);
				if( empty($trid) ){
					// TODO error notification translation couldn't be created
					$this->netsuite_connection->processMessage("<strong>The translation was created but couldn't be setup, id# {$translation_id}</strong>", false, true);
					continue;
				}
				$job_id = $this->get_job_id( $trid, $translation_id, $original_post_language_info->language_code, $lang );
				if( empty($job_id) ){
					// TODO error notification we couldn't set the translation as complete
					$this->netsuite_connection->processMessage("<strong>The translation was created but the job couldn't be created, id# {$translation_id}</strong>", false, true);
					continue;
				}
				
				$this->netsuite_connection->processMessage("Translation created/updated for <strong>{$info['locale']}</strong>");
				
				do_action('in8_product_sync_wpml_trans_saved', $trid);
			}
			// remove the filter so we can use it latter
			remove_filter('in8_product_sync_skip', [$this, 'skip_product'], 10, 2);
			$this->netsuite_connection->processMessage("Product wpml translations sync completed!", true);
		}
		
		/**
		 * Always skip the products when we're updating them to create translations
		 *
		 * @param $skip
		 * @param $id
		 *
		 * @return bool
		 */
		function skip_product($skip, $id)
		{
			return true;
		}
		
		/**
		 * Search the translation status table to find if there's there is already a translation and return it
		 *
		 * @param $trid
		 *
		 * @return bool
		 */
		function get_translation_status($trid){
			global $wpdb;
			$status = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}icl_translation_status WHERE translation_id={$trid} LIMIT 1");
			if( isset($status[0]) ){
				return $status[0];
			}
			return false;
		}
		
		/**
		 * Get the job if exists based on the status
		 *
		 * @param $status
		 *
		 * @return bool
		 */
		function get_job_from_status($status){
			if( !isset($status->rid) ) return false;
			
			global $wpdb;
			$job = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}icl_translate_job WHERE rid={$status->rid} LIMIT 1");
			if( isset($job[0]) ){
				return $job[0];
			}
			return false;
		}
		
		/**
		 * Set the translation status as completed
		 *
		 * @param $status
		 *
		 * @return bool
		 */
		const COMPLETE_STATUS_CODE = 10;
		function translation_complete($status){
			if( !isset($status->rid) ) return false;
			
			global $wpdb;
			$wpdb->update("{$wpdb->prefix}icl_translation_status", ['status' => self::COMPLETE_STATUS_CODE, 'needs_update' => 0], ['rid' => $status->rid]);
			return true;
		}
		
		/**
		 * Get job id if exists
		 *
		 * @param $trid
		 * @param $post_id
		 * @param $source_language_code
		 * @param $language_code
		 *
		 * @return bool
		 */
		function get_job_id($trid, $post_id, $source_language_code, $language_code){
			global $iclTranslationManagement, $wpml_post_translations, $wpml_translation_job_factory, $sitepress;
			$job_id = false;
			if ( $trid && $language_code ) {
				
				// Get translation status row, if this exists then it's an update so we don't need to create the job
				$status = $this->get_translation_status($trid);
				if( isset($status->rid) ) {
					$job_id = $this->get_job_from_status($status);
				}
				
				// The job doesn't exist yet we want to make sure it's not been created
				if ( ! $job_id ) {
					$job_id = $iclTranslationManagement->get_translation_job_id( $trid,
						$language_code );
					if ( ! $job_id ) {
						$blog_translators = wpml_tm_load_blog_translators();
						$args             = array(
							'lang_from' => $source_language_code,
							'lang_to'   => $language_code,
							'job_id'    => $job_id
						);
						if ( $post_id && $blog_translators->is_translator( $sitepress->get_current_user()->ID,
								$args )
						) {
							// Create the job
							$job_id = $wpml_translation_job_factory->create_local_post_job( $post_id,
								$language_code );
						}
					}
				}
			}
			if( !$status ){
				$status = $this->get_translation_status($trid);
				// Mark this product as completed
				$this->translation_complete($status);
			}
			return $job_id;
		}
		
		/**
		 * Get language code conversion from netsuite to wpml
		 *
		 * @param $code
		 *
		 * @return bool
		 */
		function get_lang_code($code){
			global $sitepress;
			$active_langs = $sitepress->get_active_languages();
			foreach($active_langs as $lang){
				if( $lang['default_locale'] == $code ){
					return $lang['code'];
				}
			}
			return false;
		}
		
	}
	
	new wc_create_update_product_translation();