<?php
	
	// If the transaction id is missing set the order to sync async
	class In8_Products_Options {
		public function __construct() {
			
			$this->show_internal_id_filter();
			
			$this->api_search_by_sku_include_draft();
			
			$this->handle_product_type_change();
		}
		
		
		function show_internal_id_filter() {
			
			add_filter( 'views_edit-product', function ( $views ) {
				
				$count = "(" . In8_Products_Options::getUnSyncedCount() . ")";
				$field = '<a href="' . admin_url( 'edit.php?post_type=product&un-synced=1' ) . '">Unsynced Products ' . $count . '</a>';
				if ( isset( $_REQUEST['un-synced'] ) ) {
					$field = 'Unsynced Products ' . $count;
				}
				
				$views['un-synced'] = $field;
				
				return $views;
				
			} );
			add_filter( 'pre_get_posts', function ( $query ) {
				global $pagenow;
				
				if ( 'edit.php' != $pagenow || ! $query->is_admin ) {
					return $query;
				}
				
				if ( isset( $_REQUEST['un-synced'] ) && isset($_REQUEST['post_type']) && $_REQUEST['post_type'] == 'product' ) {
					/* @var WP_Query */
					$meta_query = [
						[
							'key'     => 'internalid',
							'compare' => 'NOT EXISTS',
						],
					];
					
					$query->set( 'post_status', 'publish' );
					$query->set( 'meta_query', $meta_query );
				}
				
				return $query;
			} );
		}
		
		static function getUnSyncedCount() {
			$args = [
				'meta_key'       => 'internalid',
				'posts_per_page' => - 1,
				'meta_compare'   => "NOT EXISTS",
				'post_type'      => 'product',
				'post_status'    => 'publish',
				'fields'         => 'ids',
			];
			
			$wp_query = new WP_Query( $args );
			
			return $wp_query->post_count;
		}
		
		
		function api_search_by_sku_include_draft(){
			add_filter('woocommerce_api_query_args', function($default){
				
				if( isset($_REQUEST['filter']) && isset($_REQUEST['filter']['sku']) ) {
					$default['post_status'] = [ 'publish', 'draft' ];
				}
				
				return $default;
			});
			
			add_filter('woocommerce_variable_children_args', function($args){
				if( isset($_REQUEST['ns']) ){
					if( !is_array($args['post_status']) ){
						$args['post_status'] = [$args['post_status']];
					}
					$args['post_status'][] = 'draft';
				}
				
				return $args;
			});
		}
		
		
		/**
		 * This function allows us to change the product type by sending _in8_product_type on the main products endpoint
		 */
		function handle_product_type_change(){
			add_filter('woocommerce_api_dispatch_args', function($args){
				
				if( strpos($args['_route'], '/products/') !== FALSE && !empty($args['id']) ){
					if( !empty($args['data']) && !empty($args['data']['product']['_in8_product_type']) ){
						$product = wc_get_product($args['id']);
						if( $args['data']['product']['_in8_product_type'] !== $product->get_type() ) {
							$classname = WC_Product_Factory::get_classname_from_product_type( $args['data']['product']['_in8_product_type'] );
							
							if ( ! class_exists( $classname ) ) {
								$classname = 'WC_Product_Simple';
							}
							
							$product = new $classname( $args['id'] );
							$product->save();
						}
					}
				}
				
				return $args;
			});
			
		}
		
		
	}
	
	new In8_Products_Options();