<?php

    /**
     * @property NetsuiteConnection $netsuite_connection
    */
    class ProductSync {

        private $netsuite_connection;
        public static $optionsTag = 'in8_netsuite_products';

        public static function getOptionsTag() {
            return self::$optionsTag;
        }

        public function __construct() {
            //action hook for loading navigation link
            add_filter('in8_admin_tab', [$this, 'addTab']);
            //action hook for loading page content
            add_action('in8_netsuite_top_level_tabs_page', [$this, 'tabContent']);
            add_action('wp_ajax_in8_product_plugin_enabled', [$this, 'save_product_enable_form']);
            if (defined('PRODUCT_SYNC_ENABLED')) {
// Add Variation Settings
                add_action('woocommerce_product_after_variable_attributes', 'variation_settings_fields', 10, 3);
// Save Variation Settings
                add_action('woocommerce_save_product_variation', 'save_variation_settings_fields', 10, 2);
                //ajax for saving setting data
                add_action('wp_ajax_' . self::getOptionsTag(), [$this, 'save_form']);

                //action hook for determining when a new product is created
                add_action('save_post', [$this, 'productAddedUpdated'], 10, 3);

                // Used for adding/updating all woo products to NS
                add_action('admin_init', [ $this, 'addUpdateAllProducts']);

                //add admin meta box for product page
                add_action('add_meta_boxes', [$this, 'add_product_meta_box'], 10, 1);
	            add_action('admin_enqueue_scripts', function(){
		            wp_enqueue_script('admin-script', In8_Products_Plugin_URL . "inc/assets/js/admin-script.js");
		            wp_enqueue_style('admin-style', In8_Products_Plugin_URL . "inc/assets/css/admin-style.css");
	            });
                
                //ajax for pushing order to netsuite
                add_action('wp_ajax_push_product_manually', array($this, 'pushProductManually'));
                
                // add 'Sync to Netsuite' option on products bulk actions selector
				add_action('admin_footer-edit.php', [$this, 'addBulkActionsOption']);
				add_action('admin_init', [$this, 'processBulkSyncToNs']);
				add_action( 'admin_notices', [$this, 'processBulkSyncToNsNotices'] );
				
				// add the item type field to the general tab
                add_action('woocommerce_product_options_general_product_data', [$this, 'addItemTypeField']);
				add_action('save_post', [$this, 'itemTypeSave'], 10, 3);
	
				add_action('admin_enqueue_scripts', [$this, 'registerScripts']);
            }
        }

        public function save_product_enable_form() {

            if (!empty($_REQUEST['action']) && $_REQUEST['action'] == 'in8_product_plugin_enabled') {
                if (!empty($_REQUEST['options'])) {
                    $options = $_REQUEST['options'];
                } else {
                    $options = FALSE;
                }
                $option_name = $_REQUEST['action'];

                update_option($option_name, $options);

                die(json_encode([
                    'type' => 'success',
                ]));
            }

            die(json_encode([
                'type' => 'faliure',
            ]));
        }

        public function addTab($tabs) {
            $tabs[self::$optionsTag] = ['label' => "Products", "priority" => 2];
            return $tabs;
        }

        public function tabContent() {
            require_once('page-content.php');
        }

        /**
         * Create new fields for variations
         *
         */
        function variation_settings_fields($loop, $variation_data, $variation) {
            // Text Field
            woocommerce_wp_text_input(
                    array(
                        'id' => 'internalid[' . $variation->ID . ']',
                        'label' => __('Internal Id', 'woocommerce'),
                        'description' => __('Internal Id Required', 'woocommerce'),
                        'value' => get_post_meta($variation->ID, 'internalid', true)
                    )
            );
        }

        /**
         * Save new fields for variations
         *
         */
        function save_variation_settings_fields($post_id) {
            // Text Field
            $text_field = $_POST['internalid'][$post_id];
            if (!empty($text_field)) {
                update_post_meta($post_id, 'internalid', esc_attr($text_field));
            }
        }

        public static function options() {
            return get_option(self::getOptionsTag());
        }

        public function save_form() {

            $options = $_REQUEST['options'];

            update_option(self::getOptionsTag(), $options);

            die(json_encode([
                'type' => 'success',
            ]));
        }

        public function add_product_meta_box($post_type) {

            if ($post_type != "product") { //only allow products
                return;
            }

            add_meta_box('woocommerce-netsuite-product-sync', __('In8 NetSuite Synchronization', 'woocommerce-netsuite-product-sync'), [$this, 'meta_box'], 'product', 'side', 'high');
        }

        /**
         * Show the meta box for shipment info on the order page
         *
         * @access public
         */
        public function meta_box() {
            global $post;

            $product_internalID = get_post_meta($post->ID, 'internalid', TRUE);
            

            echo "<p><a class='thickbox' href='?view_in8_log_type=log&view_in8_log_object=product&view_in8_log_object_id=". $post->ID ."'>Load</a> product logs</p>";
            if (empty($product_internalID)) {
                echo "<p id='manualNSAddUpdate'><span class='replace-content'>This product has not been synced with NetSuite.</span><br/><a data-product_id='" . $post->ID . "' data-admin_ajax='" . admin_url('admin-ajax.php') . "' href='javascript:void(0)' id='manualProductPush'>Click here</a> to add it on NetSuite</p>";
            } else {
                echo "<p id='manualNSAddUpdate'><span class='replace-content'>This product has been successfully transferred to NetSuite.</span><br/><a data-product_id='" . $post->ID . "' data-admin_ajax='" . admin_url('admin-ajax.php') . "' href='javascript:void(0)' id='manualProductPush'> Click Here</a> to update it on NetSuite<p>";
            }

            echo " <div id='bowlG' style='display:none;'>
	<div id='bowl_ringG'>
		<div class='ball_holderG'>
			<div class='ballG'>
			</div>
		</div></div></div>";
        }

        private function _setNetSuiteRequest($post_id) {
            if (!class_exists('NetsuiteConnection')) { //in case of ajax
//netsuite api connection file
                require_once(WP_PLUGIN_DIR . '/in8sync/code/netsuiteconnection.php');
            }

            //netsuite connection object
            $this->netsuite_connection = new NetsuiteConnection($post_id, 'product');
        }

        public function pushProductManually() {
            $return['status'] = 0;
            $product_id = $_GET['product_id'];

            if (!empty($product_id)) {
                $productInternalId = get_post_meta($product_id, 'internalid', TRUE);

                //load required files and set logger
                $this->_setNetSuiteRequest($product_id);


                if (!empty($productInternalId)) {
                    $order_message = 'Product update';
                    $result = $this->_addUpdateProductToNetSuite($product_id, NULL);
                } else {
                    $order_message = 'Product add';
                    $result = $this->_addUpdateProductToNetSuite($product_id, NULL);
                    $productInternalId = get_post_meta($product_id, 'internalid', TRUE);
                }

                if ( !$result || empty($productInternalId)) { //order doesn't exists on Netsuite
                    $return['status'] = 0;
                    $return['message'] = "There seems to be some problem with syncing this product.";

                    if (file_exists($this->netsuite_connection->logger->log_file)) {
                        $return['message'].= "<a href='?download_in8_log_type=log&download_in8_log_object=product&download_in8_log_object_id=" . $product_id . "'> Click Here </a> to download error log file";
                    }
                } else {
                    $return['status'] = 1;
                    $return['message'] = $order_message . " request was successful";
                }

                echo json_encode($return);
                die;
            }
        }

        /**
         * This function is triggered by wordpress action hook save_post
         */
        public function productAddedUpdated($post_id, $post) {
            //Check it's not an auto save routine
            if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
                return;

            if ($post->post_type != "product" || $post->post_status == "auto-draft") { //only allow products
                return;
            }
            //load required files and set logger
            $this->_setNetSuiteRequest($post_id);
            $this->_addUpdateProductToNetSuite($post_id, $post);
        }

        /**
         * This function is responsible to send all products to NS
         * This also includes category filtering option
         */
        public function addUpdateAllProducts() {
            if (empty($_GET['importAllProducts'])) { //validate request
                return;
            }

            $args = [
                'post_status' => 'publish',
                'post_parent' => 0,
                'post_type' => 'product',
                'posts_per_page' => -1,
            ];

            $options = self::options();

            if (isset($options['categories']) && !empty($options['categories']) && $options['categories'] != -1) {
                $args['tax_query'] = [
                    [
                        'taxonomy' => 'product_cat',
                        'field' => 'id',
                        'terms' => $options['categories']
                ]];
            }

            $products = get_posts($args);
            if (!empty($products)) {
                //load required files and set logger
                $this->_setNetSuiteRequest(0);

                foreach ($products as $product) {
                    $this->_addUpdateProductToNetSuite($product->ID, $product);
                }
            }
            //redirecting to in8 main admin page
            if (wp_get_referer()) {
                wp_safe_redirect(wp_get_referer());
            } else {
                wp_safe_redirect(get_home_url() . "#in8_netsuite_products");
            }
        }

        /**
         * This function handles add/updating a product to NS
         * @param type $id
         * @param type $postobject
         */
        private function _addUpdateProductToNetSuite($id = 0, $postobject = NULL) {
            if ($id !== 0 && is_numeric($id)) {
                if( apply_filters('in8_product_sync_skip', false, $id) ){
                    return false;
                }
                
				$this->netsuite_connection->processMessage('Product sync started.');
				
                $product = new WC_Product($id);
                //if post obejct is not sent to this function
                if (is_null($postobject)) {
                    $postobject = $product->post;
                }
                $saved_map = IN8_Product_Map::getFinalMap();
                
                // Get the record type
                $recordType = $this->getProductRecordType($id);
                if( empty($recordType) ){
					$this->netsuite_connection->processMessage('Missing record type.');
					return FALSE;
                }
	
				//prepare data
				$product_data = [
                    'recordType' => $recordType,
                    //'displayname' => $postobject->post_title,
                    //'itemid' => $postobject->post_title,
                    'custitem_in8_wc_product_id' => $postobject->ID,
                ];
	
				$this->netsuite_connection->processMessage('Fields mapping.');
				foreach ($saved_map as $wp => $ns) {
                    $product_data = IN8_process_Product_Map::processField($product, $wp, $product_data, $ns);
                }

                // internalId
                $internalId = get_post_meta($id, 'internalid', TRUE);

                if (!empty($internalId)) {
                    $product_data['internalId'] = $internalId;
                }
	
				$this->netsuite_connection->processMessage('Dynamic pricing.');
				$bulkPricing = $this->getDynamicPricingForNetSuite($id);
                if (!empty($bulkPricing)) {
                    $product_data['pricingMatrix'] = $bulkPricing; //add/update product
                }
	
				$this->netsuite_connection->processMessage('NetSuite request');
	
	            $product_data = apply_filters('in8sync_product_after_data_prepared', $product_data, $id);
	
				$response = $this->netsuite_connection->doRequest($product_data);

                if ($response && isset($response['data']['id'])) {
					
                    if (!empty($product_data['internalId'])) {
                        update_post_meta($id, "internalid", $response['data']['id'], TRUE);
                    } else {
                        add_post_meta($id, "internalid", $response['data']['id'], TRUE);
                    }
	
					$this->netsuite_connection->processMessage('Product sync completed', true);
                    
					return TRUE;
                }
            }
        }

        /**
         * This function is for supporting dynamic pricing
         */
        public function getDynamicPricingForNetSuite($id) {
            $pricing = get_post_meta($id, '_pricing_rules', true);

            $_rules = [];
            if (!empty($pricing) && is_array($pricing)) {
                foreach ($pricing as $sets) {
                    if (!empty($sets['rules']) && is_array($sets['rules'])) {
                        foreach ($sets['rules'] as $rule) {
                            $rule = [
                                'priceId' => '1',
                                'priceLevel' => '2',
                                'quantityLevel' => '1',
                                'price' => $rule['amount'],
                            ];
                            $_rules[] = $rule;
                        }
                    }
                }
            }

            return $_rules;
        }
        
        public function addBulkActionsOption()
        {
            global $post_type;
            if($post_type != 'product') return;
            
            ?>
                <script type="text/javascript">
                    jQuery(document).ready(function() {
                        jQuery('<option>').val('sync_to_ns').text('<?= _e('Sync to NS')?>').appendTo("select[name='action']");
                        jQuery('<option>').val('sync_to_ns').text('<?= _e('Sync to NS')?>').appendTo("select[name='action2']");
                    });
                </script>
            <?php
        }
        public $processBulkSyncToNsNotices = 'in8_processBulkSyncToNsNotices';
        public function processBulkSyncToNs()
        {
            if( empty($_REQUEST['post_type']) ) return;
            $post_type = $_REQUEST['post_type'];
			if($post_type != 'product') return;
	
			if( !isset($_REQUEST['action']) && !isset($_REQUEST['action2']) ) return;
			if( (isset($_REQUEST['action']) && $_REQUEST['action'] != 'sync_to_ns') && (isset($_REQUEST['action2']) && $_REQUEST['action2'] != 'sync_to_ns') ) return;
	
            if( empty($_REQUEST['post']) ) return;
            $posts = $_REQUEST['post'];
            if( !is_array($posts) ) return;

            $message = '<p>Start sync</p>';
                $message .= "<ul style='padding: 0 20px;list-style-type: circle;'>";
                    foreach($posts as $post){
                        $productInternalId = get_post_meta($post, 'internalid', TRUE);
                        $addProduct = empty($productInternalId);
                        
                        // notify the user on what happened
                        $msg = '<li>';
                            $msg .= $addProduct ? "Created product <strong>{$post}</strong>" : "Updated product <strong>{$post}</strong>";
                        $msg .= '</li>';
                        
                        $this->_setNetSuiteRequest($post);
                        $this->_addUpdateProductToNetSuite($post);
                        
                        $message .= $msg;
                    }
			    $message .= "</ul>";
			$message .= '<p>End Sync</p>';
            if( !empty($message) ){
                update_option($this->processBulkSyncToNsNotices, ['message' => $message]);
            }
		}
		public function processBulkSyncToNsNotices()
        {
            $notices = get_option($this->processBulkSyncToNsNotices);
            if( !empty($notices) ){
                if( is_array($notices) ){
					printf( '<div class="notice notice-info"><p>%1$s</p></div>', $notices['message'] );
				}
            }
            update_option($this->processBulkSyncToNsNotices, []);
        }
	
	    public $itemTypes = [
		    'inventoryitem'					=> "Inventory Item",
		    'noninventoryitem'				=> "Noninventory Item",
		    'serviceitem'					=> "Service Item",
		    'serializedinventoryitem'		=> "Serialized Inventory Item",
		    'assemblyitem'					=> "Assembly Item",
		    'serializedassemblyitem'		=> "Serialized Assembly Item",
		    'kititem'						=> "Kit/Package Item",
		    'downloaditem'					=> "Download Item",
		    'lotnumberedassemblyitem'		=> "Lot Numbered Assembly Item",
		    'lotnumberedinventoryitem'		=> "Lot Numbered Inventory Item",
		    'markupitem'					=> "Markup Items",
		    'paymentitem'					=> "Payment Item",
		    'subtotalitem'					=> "Subtotal Item",
		    'otherchargeitem'				=> "Other Charge Item",
	    ];
        public $itemSubTypes = [
        ];
        public function addItemTypeField()
        {
            global $post;
            $itemType = get_post_meta($post->ID, 'in8_netsuite_recordType', true);
            $itemSubType = get_post_meta($post->ID, 'in8_netsuite_recordSubType', true);
            ?>
                <p class="form-field _tax_status_field ">
                    <label for="netsuite_recordType">Netsuite Record Type</label>
                    <select id="netsuite_recordType" name="netsuite_recordType" required class="select short" style="">
                        <option value=""></option>
						<?php foreach($this->itemTypes as $key => $title){ ?>
                            <option value="<?=$key?>" <?=$key == $itemType ? ' selected="selected" ' : ''?>><?=$title?></option>
                        <?php } ?>
                    </select>
                </p>
                <p class="form-field _tax_status_field ">
                    <label for="netsuite_recordSubType">Netsuite Record Sub Type</label>
                    <select id="netsuite_recordSubType" name="netsuite_recordSubType" class="select short" style="">
                        <option value=""></option>
						<?php foreach($this->itemSubTypes as $key => $title){ ?>
                            <option value="<?=$key?>" <?=$key == $itemSubType ? ' selected="selected" ' : ''?>><?=$title?></option>
                        <?php } ?>
                    </select>
                </p>
            <?php
        }
		public function itemTypeSave($post_id, $post) {
			//Check it's not an auto save routine
			if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
				return;
		
			if ($post->post_type != "product" || $post->post_status == "auto-draft") { //only allow products
				return;
			}
			if( !isset($_REQUEST['netsuite_recordType']) ) return;
			
			update_post_meta($post_id, 'in8_netsuite_recordType', $_REQUEST['netsuite_recordType']);
			update_post_meta($post_id, 'in8_netsuite_recordSubType', $_REQUEST['netsuite_recordSubType']);
		}
        
		public function registerScripts(){
            if( !isset($_REQUEST['page']) || $_REQUEST['page'] != 'in8sync' ) return;
			
			wp_enqueue_script('wc-chosen-js', In8_Products_Plugin_URL.'inc/assets/js/chosen.jquery.min.js', array('jquery'), null, true);
			wp_enqueue_style('wc-chosen', In8_Products_Plugin_URL.'inc/assets/css/chosen.min.css');
        }
        public function getProductRecordType($pid){
		    $type = get_post_meta($pid, 'in8_netsuite_recordType', true);
		    return $type;
        }
		
    }

    new ProductSync();

    